#!/system/bin/sh

######## BootMenu Script v0.8.6
######## Overclock.sh

export PATH=/sbin:/system/xbin:/system/bin

CONFIG_FILE="/system/bootmenu/config/overclock.conf"
MODULE_DIR="/system/bootmenu/ext/modules"
SCALING_GOVERNOR="/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor"
SCALING_MIN="/sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq"
SCALING_MAX="/sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq"

#############################################################
# Parameters Load
#############################################################

##### Default Param
# enable 0
# load_all 0
# scaling 2
# clk1 300
# clk2 600
# clk3 900
# clk4 1200
# vsel1 20
# vsel2 32
# vsel3 44
# vsel4 60
# con_up_threshold 80
# con_down_threshold 20
# con_freq_step 5
# con_sampling_rate 160000
# int_min_sample_rate 50000
# ond_up_threshold 86
# ond_sampling_rate 50000
# smt_min_cpu_load 10
# smt_max_cpu_load 100
# smt_awake_min_freq 200000
# smt_sleep_max_freq 200000
# smt_up_min_freq 1200000
# smt_wakeup_freq 1200000
# smt_ramp_up_step 200000

param_load()
{
  for CONF in $(sed -e 's/^\([^ ]*\) \(.*\)/\1=\2/g' $CONFIG_FILE); do
    export $CONF
  done
}

#############################################################
# Get Address
#############################################################

get_address()
{
  cpufreq_table=`grep -e omap2_clk_init_cpufreq_table /proc/kallsyms | sed -e "s/\([0-9A-Fa-f]\{8\}\).*/\1/"`
  stats_update=`grep -e cpufreq_stats_update /proc/kallsyms | sed -e "s/\([0-9A-Fa-f]\{8\}\).*/\1/"`
  nr_running=`grep -e nr_running /proc/kallsyms | sed -e "s/\([0-9A-Fa-f]\{8\}\).*/\1/"`
}

#############################################################
# Install Module
#############################################################

install_module()
{
  # load module
  insmod $MODULE_DIR/overclock_defy.ko omap2_clk_init_cpufreq_table_addr=0x$cpufreq_table
  # set cpufreq_stats_update_addr
  echo 0x$stats_update > /proc/overclock/cpufreq_stats_update_addr

  if [ $load_all -eq 1 ]; then
    insmod $MODULE_DIR/cpufreq_ondemand.ko
    insmod $MODULE_DIR/cpufreq_performance.ko
    insmod $MODULE_DIR/cpufreq_conservative.ko
    insmod $MODULE_DIR/cpufreq_interactive.ko nr_running_addr=0x$nr_running
    insmod $MODULE_DIR/cpufreq_powersave.ko
    insmod $MODULE_DIR/symsearch.ko
    insmod $MODULE_DIR/cpufreq_smartass.ko
    insmod $MODULE_DIR/cpufreq_userspace.ko
  fi
}

#############################################################
# Set Scaling
#############################################################

set_scaling()
{
  case "$scaling" in
    "0" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_conservative.ko
      fi
      echo "conservative" > $SCALING_GOVERNOR

      echo $con_sampling_rate > /sys/devices/system/cpu/cpu0/cpufreq/conservative/sampling_rate
      echo $con_freq_step > /sys/devices/system/cpu/cpu0/cpufreq/conservative/freq_step
      echo $con_up_threshold > /sys/devices/system/cpu/cpu0/cpufreq/conservative/up_threshold
      echo $con_down_threshold > /sys/devices/system/cpu/cpu0/cpufreq/conservative/down_threshold
    ;;
    "1" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_interactive.ko nr_running_addr=0x$nr_running
      fi
      echo "interactive" > $SCALING_GOVERNOR

      echo $int_min_sample_rate > /sys/devices/system/cpu/cpu0/cpufreq/interactive/min_sample_time
    ;;
    "2" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_ondemand.ko
      fi
      echo "ondemand" > $SCALING_GOVERNOR

      echo $ond_sampling_rate > /sys/devices/system/cpu/cpu0/cpufreq/ondemand/sampling_rate
      echo $ond_up_threshold > /sys/devices/system/cpu/cpu0/cpufreq/ondemand/up_threshold
    ;;
    "3" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_performance.ko
      fi
      echo "performance" > $SCALING_GOVERNOR
    ;;
    "4" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_powersave.ko
      fi
      echo "powersave" > $SCALING_GOVERNOR
    ;;
    "5" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/symsearch.ko
        insmod $MODULE_DIR/cpufreq_smartass.ko
      fi
      echo "smartass" > $SCALING_GOVERNOR

      echo $smt_min_cpu_load > /sys/devices/system/cpu/cpu0/cpufreq/smartass/min_cpu_load
      echo $smt_max_cpu_load > /sys/devices/system/cpu/cpu0/cpufreq/smartass/max_cpu_load
      echo $smt_awake_min_freq > /sys/devices/system/cpu/cpu0/cpufreq/smartass/awake_min_freq
      echo $smt_sleep_max_freq > /sys/devices/system/cpu/cpu0/cpufreq/smartass/sleep_max_freq
      echo $smt_up_min_freq > /sys/devices/system/cpu/cpu0/cpufreq/smartass/up_min_freq
      echo $smt_wakeup_freq > /sys/devices/system/cpu/cpu0/cpufreq/smartass/sleep_wakeup_freq
      echo $smt_ramp_up_step > /sys/devices/system/cpu/cpu0/cpufreq/smartass/ramp_up_step
    ;;
    "6" )
      if [ $load_all -eq 0 ]; then
        insmod $MODULE_DIR/cpufreq_userspace.ko
      fi
      echo "userspace" > $SCALING_GOVERNOR
    ;;
     * )

    ;;
  esac
}

#############################################################
# Set Clock Table
#############################################################

set_overclock_table()
{
  saf_count=`cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies | wc -w`

  if [ $saf_count -gt 3 ]; then
    echo "4 ${clk4}000000 $vsel4" > /proc/overclock/mpu_opps
    echo "3 ${clk3}000000 $vsel3" > /proc/overclock/mpu_opps
    echo "2 ${clk2}000000 $vsel2" > /proc/overclock/mpu_opps
    echo "1 ${clk1}000000 $vsel1" > /proc/overclock/mpu_opps

    echo "0 ${clk4}000" > /proc/overclock/freq_table
    echo "1 ${clk3}000" > /proc/overclock/freq_table
    echo "2 ${clk2}000" > /proc/overclock/freq_table
    echo "3 ${clk1}000" > /proc/overclock/freq_table
  else
    echo "3 ${clk3}000000 $vsel3" > /proc/overclock/mpu_opps
    echo "2 ${clk2}000000 $vsel2" > /proc/overclock/mpu_opps
    echo "1 ${clk1}000000 $vsel1" > /proc/overclock/mpu_opps

    echo "0 ${clk3}000" > /proc/overclock/freq_table
    echo "1 ${clk2}000" > /proc/overclock/freq_table
    echo "2 ${clk1}000" > /proc/overclock/freq_table
  fi
}

#############################################################
# Set Scaling Range
#############################################################

set_scaling_range()
{
  echo "${clk1}000" > $SCALING_MIN

  if [ $saf_count -gt 3 ]; then
    echo "${clk4}000" > $SCALING_MAX
  else
    echo "${clk3}000" > $SCALING_MAX
  fi
}

#############################################################
# Main Scrpit
#############################################################

if [ -e $CONFIG_FILE ]; then
  param_load
  if [ $enable -eq 1 ]; then
    get_address
    install_module
    set_overclock_table
    set_scaling_range
    set_scaling
  fi
fi